package se.chalmers.cs.gf.tramdemo;

import se.gu.ling.trindikit.oaa.common.*;

import com.sri.oaa2.icl.*;

import se.chalmers.cs.gf.abssyn.Tree;
import se.chalmers.cs.gf.oaa.IclTermToTree;

import se.chalmers.cs.gf.tramdemo.query.Query;
import se.chalmers.cs.gf.tramdemo.query.TreeToTyped;

import se.chalmers.cs.gf.util.FileUtil;

import java.io.IOException;
import java.util.List;
import java.util.Properties;

public class QueryInterpreterAgent extends OAAAgent {

        /** The name of this agent. */
        static final String name = "QueryInterpreterAgent";

        /** The only state of this agent. */
        static final int RUNNING = 0;

        public QueryInterpreterAgent(InterpretQuery i) {
                super(RUNNING,1);

                addSolver(RUNNING, new InterpretQuerySolver(i));
        }

        private static class InterpretQuerySolver extends OAASolver {

                private static String solvableString = "interpret_query(Query,ShortestPathQuery)";
        
                private InterpretQuery interpreter;

                public InterpretQuerySolver(InterpretQuery interpreter) {
                        super(solvableString);
                        this.interpreter = interpreter;
                }

                public boolean solve(IclTerm goal, IclList params, IclList answers) {
                        IclTerm queryTerm = goal.getTerm(0);
                        Tree queryTree = IclTermToTree.iclTermToTree(queryTerm);
                        Query q = TreeToTyped.getInstance().toQuery(queryTree);
                        List<ShortestPathQuery> rs = interpreter.interpretQuery(q);
                        for (ShortestPathQuery sq : rs) {
                                IclTerm sqTerm = new IclStruct("shortest_path_query", 
                                                               new IclStr(sq.getFrom()),
                                                               new IclStr(sq.getTo()));
                                answers.add(new IclStruct("interpret_query", queryTerm, sqTerm));
                        }

                        return true;
                }
        }


        public static void main(String[] args) throws IOException {
                Properties cityProps = FileUtil.readPropertiesResource(MapAgent.class, "/gbg.properties");

                double clickThreshold = Double.parseDouble(cityProps.getProperty("clickThreshold"));
                String stopHere = cityProps.getProperty("stopHere");

                String graphResource = cityProps.getProperty("graph");
                TransportNetwork tn = DotToGraph.fromDotResource(graphResource);
                CoordMap<String> coordMap = tn.getCoordMap(clickThreshold);
                
                QueryInterpreterAgent agent = new QueryInterpreterAgent(new InterpretQuery(coordMap, stopHere));

                if (!agent.register(name, args))
                        System.err.println(name + ": register() failed");
        }

}
